#!/bin/python

from yahoo_fantasy_api import yahoo_api
import objectpath


class Team:
    def __init__(self, sc, team_key):
        """Class initializer

        :param sc: Session context for oauth
        :type sc: OAuth2 from yahoo_oauth
        :param team_key: Team key identifier to find the matchups for
        :type team_key: str
        """
        self.sc = sc
        self.team_key = team_key

    def matchup(self, week, data_gen=yahoo_api.get_matchup_raw):
        """Return the team of the matchup my team is playing in a given week

        :param week: Week number to find the matchup for
        :type week: int
        :param data_gen: Optional data generation function.  Only used for
           testing purposes to avoid call-outs to Yahoo!  The function accepts
           3 parameters: sc (session context), team_key, week
        :type data_gen: function
        :return: Team key of the opponent

        >>> tm.matchup(3)
        388.l.27081.t.9
        """
        t = objectpath.Tree(data_gen(self.sc, self.team_key, week))
        json = t.execute('$..matchups..(team_key)')
        for k in json:
            if 'team_key' in k:
                this_team_key = k['team_key']
                if this_team_key != self.team_key:
                    return this_team_key
        raise RuntimeError("Could not find opponent")

    def roster(self, week, data_gen=yahoo_api.get_roster_raw):
        """Return the team roster for a given week

        :param week: Week number of the roster to get
        :type week: int
        :param data_gen: Optional data generation function.  Only used for
           testing purposes to avoid call-outs to Yahoo!  The function accepts
           3 parameters: sc (session context), team_key, week
        :type data_gen: function
        :return: Array of players.  Each entry is a dict with the following
        fields: player_id, name, position_type, eligible_positions,
        selected_position

        >>> tm.roster(3)
        [{'player_id': 8578, 'name': 'John Doe', 'position_type': 'B',
         'eligible_positions': ['C','1B'], 'selected_position': 'C'},
         {'player_id': 8967, 'name': 'Joe Baseball', 'position_type': 'B',
         'eligible_positions': ['SS'], 'selected_position': 'SS'},
         {'player_id': 9961, 'name': 'Ed Reliever', 'position_type': 'P',
         'eligible_positions': ['RP']}]
        """
        t = objectpath.Tree(data_gen(self.sc, self.team_key, week))
        it = t.execute('''
                        $..(player_id,full,position_type,eligible_positions,
                            selected_position)''')

        def _compact_selected_pos(j):
            return j["selected_position"][1]["position"]

        def _compact_eligible_pos(j):
            compact_pos = []
            for p in j["eligible_positions"]:
                compact_pos.append(p['position'])
            return compact_pos

        roster = []
        try:
            while True:
                roster.append({"player_id": int(next(it)["player_id"]),
                               "name": next(it)["full"],
                               "position_type": next(it)["position_type"],
                               "eligible_positions":
                               _compact_eligible_pos(next(it)),
                               "selected_position":
                               _compact_selected_pos(next(it))})
        except StopIteration:
            pass
        return roster



#!/bin/python

from yahoo_fantasy_api import yahoo_api, league
import objectpath


class Game:
    def __init__(self, sc, code):
        """Class initializer

        :param sc: OAuth2 session context from yahoo_oauth
        :type sc: OAuth2
        :param code: Sport code (mlb, nhl, etc)
        :type code: str.
        """
        self.sc = sc
        self.code = code

    def to_league(self, league_id):
        """Construct a League object from a Game

        :param league_id: League ID of the new League to construct
        :type league_id: str
        :return: League object
        """
        return league.League(self.sc, league_id)

    def league_ids(self, year=None, data_gen=yahoo_api.get_teams_raw):
        """Return the Yahoo! league IDs that the current user played in

        :param year: Optional year, used to filter league IDs returned.
        :type year: int
        :param data_gen: Optional data generation function.  This exists for
            test purposes to allow for test dependency injection.  The default
            value for this parameter is the Yahoo! API.
        :returns: List of league ids
        """
        t = objectpath.Tree(data_gen(self.sc))
        jfilter = t.execute('$..(team_key,season,code)')
        league_applies = False
        ids = []
        for row in jfilter:
            # We'll see two types of rows that come out of objectpath filter.
            # A row that has the season/code, then all of the leagues that it
            # applies too.  Check if the subsequent league applies each time we
            # get the season/code pair.
            if 'season' in row and 'code' in row:
                league_applies = row['code'] == self.code
                if league_applies is True and year is not None:
                    league_applies = int(row['season']) == int(year)
            elif league_applies:
                assert('team_key' in row)
                ids.append(self._extract_id_from_team_key(row['team_key']))
        # Return leagues in deterministic order
        ids.sort()
        return ids

    def _extract_id_from_team_key(self, t):
        """Given a team key, extract just the league id from it

        A team key is defined as:
            <game#>.l.<league#>.t.<team#>
        """
        assert(t.find(".t.") > 0), "Doesn't look like a valid team key: " + t
        return t[0:t.find(".t.")]



#!/bin/python

import json

YAHOO_ENDPOINT = 'https://fantasysports.yahooapis.com/fantasy/v2'


def get(sc, uri):
    """Send an API request to the URI and return the response as JSON

    Args:
        sc - OAuth2 session context
        uri - URI of the API to call

    Return:
        JSON document of the response
    """
    response = sc.session.get("{}/{}".format(YAHOO_ENDPOINT, uri),
                              params={'format': 'json'})
    jresp = response.json()
    if "error" in jresp:
        raise RuntimeError(json.dumps(jresp))
    return jresp


def get_teams_raw(sc):
    """Return the raw JSON when requesting the logged in players teams.

    :param sc: OAuth2 session context from yahoo_oauth
    :type sc: OAuth2
    :return: JSON document of the request.
    """
    return get(sc, "users;use_login=1/games/teams")


def get_standings_raw(sc, league_id):
    """Return the raw JSON when requesting standings for a league.

    :param sc: Session context for oauth
    :type sc: OAuth2 from yahoo_oauth
    :param league_id: League ID to get the standings for
    :type league_id: str
    :return: JSON document of the request.
    """
    return get(sc, "league/{}/standings".format(league_id))


def get_settings_raw(sc, league_id):
    """Return the raw JSON when requesting settings for a league.

    :param sc: Session context for oauth
    :type sc: OAuth2 from yahoo_oauth
    :param league_id: League ID to get the standings for
    :type league_id: str
    :return: JSON document of the request.
    """
    return get(sc, "league/{}/settings".format(league_id))


def get_matchup_raw(sc, team_key, week):
    """Return the raw JSON when requesting match-ups for a team

    :param sc: Session context for oauth
    :type sc: OAuth2 from yahoo_oauth
    :param team_key: Team key identifier to find the matchups for
    :type team_key: str
    :param week: What week number to request the matchup for?
    :type week: int
    :return: JSON of the request
    """
    return get(sc, "team/{}/matchups;weeks={}".format(team_key, week))


def get_roster_raw(sc, team_key, week):
    """Return the raw JSON when requesting a team's roster

    :param sc: Session context for oauth
    :type sc: OAuth2 from yahoo_oauth
    :param team_key: Team key identifier to find the matchups for
    :type team_key: str
    :param week: What week number to request the matchup for?
    :type week: int
    :return: JSON of the request
    """
    return get(sc, "team/{}/roster/players;week={}".format(team_key, week))



#!/bin/python

from yahoo_fantasy_api import yahoo_api, team
import objectpath


class League:
    def __init__(self, sc, league_id):
        """Class initializer

        :param sc: Session context for oauth
        :type sc: OAuth2 from yahoo_oauth
        :param league_id: League ID to setup this class for.  All API requests
            will be for this league.
        :type code: str.
        """
        self.sc = sc
        self.league_id = league_id

    def to_team(self, team_key):
        """Construct a Team object from a League

        :param team_key: Team key of the new Team object to construct
        :type team_key: str
        :return: Team object
        """
        return team.Team(self.sc, team_key)

    def standings(self, data_gen=yahoo_api.get_standings_raw):
        """Return the standings of the league id

        :param data_gen: Optional data generation function.  This exists for
            test purposes so that we don't have to call-out to Yahoo!
        :type data_type: Function
        :return: An ordered list of the teams in the standings.  First entry is
            the first place team.

        >>> lg.standings()
        ['Liz & Peter's Twins', 'Lumber Kings', 'Proj. Matt Carpenter']
        """
        json = data_gen(self.sc, self.league_id)
        team_json = \
            json['fantasy_content']["league"][1]["standings"][0]["teams"]
        standings = []
        for i in range(team_json["count"]):
            team = team_json[str(i)]["team"][0]
            standings.append(team[2]['name'])
        return standings

    def settings(self, data_gen=yahoo_api.get_settings_raw):
        """Return the league settings

        :param data_gen: Optional data generation function.  This exists for
            test purposes so that we don't have to call-out to Yahoo!
        :type data_type: Function

        >>> lg.setings()
        {'name': "Buck you're next!", 'scoring_type': 'head',
        'start_week': '1', 'current_week': 1, 'end_week': '24',
        'start_date': '2019-03-20', 'end_date': '2019-09-22',
        'game_code': 'mlb', 'season': '2019'}
        """
        json = data_gen(self.sc, self.league_id)
        t = objectpath.Tree(json)
        settings_to_return = """
        name, scoring_type,
        start_week, current_week, end_week,start_date, end_date,
        game_code, season
        """
        return t.execute('$.fantasy_content.league.({})[0]'.format(
            settings_to_return))

    def stat_categories(self, data_gen=yahoo_api.get_settings_raw):
        """Return the stat categories for a league

        :param data_gen: Optional data generation function.  Only used for
           testing purposes to avoid call-outs to Yahoo!
        :type data_gen: function
        :returns: An array of dicts.  Each dict will have the stat name along
            with the position type ('B' for batter or 'P' for pitcher).

        >>> lg.stat_categories('370.l.56877')
        [{'display_name': 'R', 'position_type': 'B'}, {'display_name': 'HR',
        'position_type': 'B'}, {'display_name': 'W', 'position_type': 'P'}]
        """
        t = objectpath.Tree(data_gen(self.sc, self.league_id))
        json = t.execute('$..stat_categories..stat')
        simple_stat = []
        for s in json:
            # Omit stats that are only for display purposes
            if 'is_only_display_stat' not in s:
                simple_stat.append({"display_name": s["display_name"],
                                    "position_type": s["position_type"]})
        return simple_stat

    def team_key(self, data_gen=yahoo_api.get_teams_raw):
        """Return the team_key for logged in users team in this league

        :param data_gen: Optional data generation function.  Only used for
           testing purposes to avoid call-outs to Yahoo!
        :type data_gen: function

        >>> lg.team_key
        388.l.27081.t.5
        """
        t = objectpath.Tree(data_gen(self.sc))
        json = t.execute('$..(team_key)')
        for t in json:
            if t['team_key'].startswith(self.league_id):
                return t['team_key']
